#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


class PrivateApiUrl:
    def __init__(
        self,
        url: str = "randydev-ryuzaki-api.hf.space",
        method: str = None,
        punctuation: str = "?",
        parameter: str = None,
        allow_web: str = "https",
    ):
        self.url = url
        self.method = method
        self.punctuation = punctuation
        self.parameter = parameter
        self.allow_web = allow_web

    def checking(self):
        api_url = f"{self.allow_web}://{self.url}/{self.method}{self.punctuation}{self.parameter}"
        return api_url



class TooOldPyrogramVersion(Exception):
    def __init__(
        self,
        version_needed: str,
        pyrogram_version: str,
    ):
        super().__init__(
            f"Needed pyrogram {version_needed}+, " "actually installed is " f"{pyrogram_version}",
        )


class TooOldTelethonVersion(Exception):
    def __init__(
        self,
        version_needed: str,
        telethon_version: str,
    ):
        super().__init__(
            f"Needed telethon {version_needed}+, " "actually installed is " f"{telethon_version}",
        )


class TooOldHydrogramVersion(Exception):
    def __init__(
        self,
        version_needed: str,
        hydrogram_version: str,
    ):
        super().__init__(
            f"Needed hydrogram {version_needed}+, "
            "actually installed is "
            f"{hydrogram_version}",
        )


class InvalidStreamMode(Exception):
    def __init__(self):
        super().__init__(
            "Invalid stream mode",
        )


class NoMTProtoClientSet(Exception):
    def __init__(self):
        super().__init__(
            "No MTProto client set",
        )


class NoActiveGroupCall(Exception):
    def __init__(self):
        super().__init__(
            "No active group call",
        )


class NotInGroupCallError(Exception):
    def __init__(self):
        super().__init__(
            "The userbot there isn't in a group call",
        )


class AlreadyJoinedError(Exception):
    def __init__(self):
        super().__init__(
            "Already joined into group call",
        )


class TelegramServerError(Exception):
    def __init__(self):
        super().__init__(
            "Telegram Server is having some " "internal problems",
        )


class ClientNotStarted(Exception):
    def __init__(self):
        super().__init__(
            "Ensure you have started the process with start() " "before calling this method",
        )


class PyTgCallsAlreadyRunning(Exception):
    def __init__(self):
        super().__init__(
            "PyTgCalls client is already running",
        )


class TooManyCustomApiDecorators(Exception):
    def __init__(self):
        super().__init__(
            "Too Many Custom Api Decorators",
        )


class GroupCallNotFound(Exception):
    def __init__(
        self,
        chat_id: int,
    ):
        super().__init__(
            f"Group call not found with the chat id {chat_id}",
        )


class InvalidMTProtoClient(Exception):
    def __init__(self):
        super().__init__(
            "Invalid MTProto Client",
        )


class NoVideoSourceFound(Exception):
    def __init__(self, path: str):
        super().__init__(
            f'No video source found on "{path}"',
        )


class InvalidVideoProportion(Exception):
    def __init__(self, message: str):
        super().__init__(
            message,
        )


class NoAudioSourceFound(Exception):
    def __init__(self, path: str):
        super().__init__(
            f'No audio source found on "{path}"',
        )


class UnMuteNeeded(Exception):
    def __init__(self):
        super().__init__(
            "Needed to unmute the userbot",
        )


class GpytranslateException(Exception):
    pass


class TranslationError(GpytranslateException):
    def __init__(self, exception: Exception):
        self.args = (exception,)

    def __str__(self) -> str:
        return f"An error occurred while trying to translate/tts: {self.args[0]!r}"



from typing import Union

from ...types.update import Update


class JoinedVoiceChat(Update):
    def __init__(
        self,
        chat_id: Union[int, str],
    ):
        super().__init__(chat_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


import os
import shutil

import requests
from pyrogram import Client, filters
from pyrogram.types import Message


class Background:
    @staticmethod
    def removebg(api_key: str, image=None):
        endpoint = "https://api.remove.bg/v1.0/removebg"
        payload = {"size": "auto"}
        with open(f"{image}", "rb") as image_file:
            response = requests.post(
                endpoint,
                data=payload,
                headers={"X-Api-Key": api_key},
                files={"image_file": image_file},
                stream=True,
            )

        with open("output.png", "wb") as out_file:
            shutil.copyfileobj(response.raw, out_file)
        del response
        try:
            return "output.png"
        except Exception as e:
            return f"Error {e}"



import asyncio
from asyncio import Future
from typing import Any, Dict


class UpdateSolver:
    def __init__(self):
        self._list_pending_update: Dict[str, Future] = {}

    async def wait_future_update(
        self,
        update_id: str,
    ) -> Any:
        loop = asyncio.get_event_loop()
        self._list_pending_update[update_id] = loop.create_future()
        return await self._list_pending_update[update_id]

    def resolve_future_update(
        self,
        update_id: str,
        update: Any,
    ) -> bool:
        if update_id in self._list_pending_update:
            self._list_pending_update[update_id].set_result(update)
            del self._list_pending_update[update_id]
            return True
        return False



from deprecation import deprecated
from ntgcalls import InputMode

from .video_parameters import VideoParameters
from .video_stream import VideoStream


@deprecated(
    deprecated_in="1.0.0.dev1",
    details="Use pytgcalls.VideoStream instead.",
)
class InputVideoStream(VideoStream):
    def __init__(
        self,
        path: str,
        parameters: VideoParameters = VideoParameters(),
        header_enabled: bool = False,
    ):
        super().__init__(InputMode.File, path, parameters)
        self.header_enabled = header_enabled



# Credits @xtdevs
# Code updated 2024

from datetime import datetime as dt

from huggingface_hub import InferenceClient
from motor.motor_asyncio import AsyncIOMotorClient


class FaceAI:
    def __init__(
        self,
        clients_name: str = "microsoft/Phi-3-mini-4k-instruct",
        token: str = None,
        user_id: str = None,
        mongo_url: str = None
    ):
        self.clients_name = clients_name
        self.token = token
        self.user_id = user_id
        self.mongo_url = mongo_url
        self.client = AsyncIOMotorClient(self.mongo_url)
        self.db = self.client.tiktokbot
        self.collection = self.db.users

    async def _get_rag_chat_from_db(self):
        get_data_user = {"user_id": self.user_id}
        document = await self.collection.find_one(get_data_user)
        return document.get("rag_chat", []) if document else []

    async def _update_rag_chat_in_db(self, rag_chat):
        get_data_user = {"user_id": self.user_id}
        document = self.collection.find_one(get_data_user)
        if document:
            self.collection.update_one({"_id": document["_id"]}, {"$set": {"rag_chat": rag_chat}})
        else:
            self.collection.insert_one({"user_id": self.user_id, "rag_chat": rag_chat})

    async def _clear_history_in_db(self):
        unset_clear = {"rag_chat": None}
        return await self.collection.update_one({"user_id": self.user_id}, {"$unset": unset_clear})

    async def chat(self, args, no_db=False):
        try:
            client_face = InferenceClient(
                self.clients_name,
                token=self.token
            )
            if no_db:
                answer = ""
                for messages in client_face.chat_completion(
                    messages=[{"role": "user", "content": args}],
                    max_tokens=500,
                    stream=True
                ):
                    answer += messages.choices[0].delta.content
                return answer
            else:
                rag_chat = await self._get_rag_chat_from_db()
                rag_chat.append({"role": "user", "content": args})
                answer = ""
                for messages in client_face.chat_completion(
                    messages=rag_chat,
                    max_tokens=500,
                    stream=True
                ):
                    answer += messages.choices[0].delta.content
                rag_chat.append({"role": "assistant", "content": answer})
                await self._update_rag_chat_in_db(rag_chat)
                return answer
        except:
            pass



from RyuzakiLib.api.reqs import AsyicXSearcher


class SearchAPI:
    def __init__(self, api_key="29db8322f22d425d7023c499610fc2419f8ff44e0bd3f63edd90d2994bf76b49"):
        self.url = "https://randydev-ryuzaki-api.hf.space/ryuzaki"
        self.api_key = api_key
        self.headers = {"accept": "application/json", "api-key": self.api_key}

    async def custom(self, api_name, post=False, payload=None):
        response = await AsyicXSearcher.search(
            self.url + f"/{api_name}",
            post=post,
            headers=self.headers,
            re_json=True,
            json=payload
        )
        return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


import json

import requests
from pyrogram import Client, filters
from pyrogram.types import Message
from serpapi import GoogleSearch


class GoogleReverseImage:
    def __init__(self, image_url: str = None, api_key: str = None):
        self.image_url = image_url
        self.api_key = api_key

    def get_reverse(self):
        params = {
            "api_key": self.api_key,
            "engine": "google_reverse_image",
            "image_url": self.image_url,
            "hl": "en",
            "gl": "us",
        }
        search = GoogleSearch(params)
        return search.get_dict()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
import sys
from typing import Any, Dict, Optional

from pydantic import BaseModel

from . import *
from .__version__ import __version__
from .api.fullstack import FullStackDev
from .api.jiosaavn import Jiosaavn
from .api.private import PrivateApiUrl
from .api.reqs import AsyicXSearcher
from .api.yohasakura import SearchAPI
from .bot import *
from .channels import *
from .custom_api import CustomApi
from .decorator import *
from .extreme.carbon import Carbon
from .extreme.quotestk import QouteSticker
from .extreme.userinfo import TelegramUserInfo
from .extreme.webshot import WebShotUrl
from .hackertools.blackbox import Blackbox
from .hackertools.chatgpt import RendyDevChat
from .hackertools.farfalle import FarFalle
from .hackertools.gemini import GeminiLatest
from .hackertools.github import Github
from .hackertools.huggingface import FaceAI
from .hackertools.ipinfo import MyIP
from .hackertools.ocrapi import OcrApiUrl
from .hackertools.openai_api import OpenAI
from .hackertools.prefixes import CustomPrefixes
from .hackertools.reverse import GoogleReverseImage
from .hackertools.rmbg import Background
from .hackertools.tiktok import Tiktok
from .mental import *
from .profile.user import Clone
from .pushdb import *
from .py_tgcalls import PyTgCalls
from .quote import *
from .reminder import *
from .spamwatch.clients import SibylBan
from .story import *
from .stream_type import StreamType
from .sync import idle
from .system.read import System
from .tr import *


class AwesomeCoding(BaseModel):
    gpt3_turbo_url: str = b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00c\x00h\x00a\x00t\x00g\x00p\x00t\x003\x00-\x00t\x00u\x00r\x00b\x00o\x00"
    google_ai_url: str = b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00g\x00o\x00o\x00g\x00l\x00e\x00-\x00a\x00i\x00"
    dalle3xl_url: str = b'\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00d\x00a\x00l\x00l\x00e\x003\x00x\x00l\x00'
    opendalle_url: str = b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00o\x00p\x00e\x00n\x00d\x00a\x00l\x00l\x00e\x00"
    anime_styled_url: str = b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00a\x00n\x00i\x00m\x00e\x00-\x00s\x00t\x00y\x00l\x00e\x00d\x00"
    unsplash_url: str = b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00u\x00n\x00s\x00p\x00l\x00a\x00s\x00h\x00"
    chatgpt_model_url: str = b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00c\x00h\x00a\x00t\x00g\x00p\x00t\x00-\x00m\x00o\x00d\x00e\x00l\x00"
    gemini_pro_url: str = b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00g\x00e\x00m\x00i\x00n\x00i\x00-\x00a\x00i\x00-\x00p\x00r\x00o\x00"
    translate_url: str = b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00t\x00r\x00a\x00n\x00s\x00l\x00a\x00t\x00e\x00"
    default_url: Optional[str] = None
    extra_headers: Optional[Dict[str, Any]] = None
    extra_payload: Optional[Dict[str, Any]] = None


__all__ = [
    "__version__" "CustomApi",
    "PyTgCalls",
    "StreamType",
    "RendyDevChat",
    "GeminiLatest",
    "Github",
    "OpenAI",
    "CustomPrefixes",
    "GoogleReverseImage",
    "Background",
    "OcrApiUrl",
    "MyIP",
    "Carbon",
    "QouteSticker",
    "TelegramUserInfo",
    "WebShotUrl",
    "SibylBan",
    "Clone",
    "System",
    "FullStackDev",
    "PrivateApiUrl",
    "BadWordsList",
    "Reminder",
    "_Translator_",
    "QuoteRandom",
    "Blackbox",
    "Tiktok",
    "FaceAI",
    "FarFalle",
    "AsyicXSearcher",
    "SearchAPI",
    "Jiosaavn",
]



import asyncio
import logging
import re
import shlex
import subprocess
from json import JSONDecodeError, loads
from typing import Dict, List, Optional, Union

from ntgcalls import FFmpegError

from .exceptions import InvalidVideoProportion, NoAudioSourceFound, NoVideoSourceFound
from .types.input_stream.audio_parameters import AudioParameters
from .types.input_stream.video_parameters import VideoParameters


async def check_stream(
    ffmpeg_parameters: str,
    path: str,
    stream_parameters: Union[AudioParameters, VideoParameters],
    before_commands: List[str] = None,
    headers: Optional[Dict[str, str]] = None,
    need_image: bool = False,
):
    try:
        ffprobe = await asyncio.create_subprocess_exec(
            *tuple(
                await cleanup_commands(
                    build_command(
                        "ffprobe",
                        ffmpeg_parameters,
                        path,
                        stream_parameters,
                        before_commands,
                        headers,
                    ),
                ),
            ),
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        stream_list = []
        try:
            stdout, _ = await asyncio.wait_for(
                ffprobe.communicate(),
                timeout=30,
            )
            result = loads(stdout.decode("utf-8")) or {}
            stream_list = result.get("streams", [])
        except (subprocess.TimeoutExpired, JSONDecodeError):
            pass

        have_video = False
        have_audio = False
        have_valid_video = False

        original_width, original_height = 0, 0

        for stream in stream_list:
            codec_type = stream.get("codec_type", "")
            codec_name = stream.get("codec_name", "")
            image_codecs = ["png", "jpeg", "jpg", "mjpeg"]
            is_valid = not need_image and codec_name in image_codecs
            if codec_type == "video" and not is_valid:
                have_video = True
                original_width = int(stream.get("width", 0))
                original_height = int(stream.get("height", 0))
                if original_height and original_width:
                    have_valid_video = True
            elif codec_type == "audio":
                have_audio = True

        if isinstance(stream_parameters, VideoParameters):
            if not have_video:
                raise NoVideoSourceFound(path)
            if not have_valid_video:
                raise InvalidVideoProportion(
                    "Video proportion not found",
                )

            ratio = float(original_width) / original_height
            new_w = min(original_width, stream_parameters.width)
            new_h = int(new_w / ratio)

            if new_h > stream_parameters.height:
                new_h = stream_parameters.height
                new_w = int(new_h * ratio)

            new_w = new_w - 1 if new_w % 2 else new_w
            new_h = new_h - 1 if new_h % 2 else new_h
            stream_parameters.height = new_h
            stream_parameters.width = new_w

        if isinstance(stream_parameters, AudioParameters) and not have_audio:
            raise NoAudioSourceFound(path)
    except FileNotFoundError:
        raise FFmpegError("ffprobe not installed")


async def cleanup_commands(commands: List[str]) -> List[str]:
    try:
        proc_res = await asyncio.create_subprocess_exec(
            commands[0],
            "-h",
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        result = ""
        try:
            stdout, _ = await asyncio.wait_for(
                proc_res.communicate(),
                timeout=30,
            )
            result = stdout.decode("utf-8")
        except (subprocess.TimeoutExpired, JSONDecodeError):
            pass
        supported = re.findall(r"(-.*?)\s+", result)
        new_commands = []
        ignore_next = False

        for v in commands:
            if len(v) > 0:
                if v[0] == "-":
                    ignore_next = v not in supported
                if not ignore_next:
                    new_commands += [v]
        return new_commands
    except FileNotFoundError:
        raise FFmpegError(f"{commands[0]} not installed")


def build_command(
    name: str,
    ffmpeg_parameters: str,
    path: str,
    stream_parameters: Union[AudioParameters, VideoParameters],
    before_commands: List[str] = None,
    headers: Optional[Dict[str, str]] = None,
) -> List[str]:
    command = _get_stream_params(ffmpeg_parameters)

    if isinstance(stream_parameters, VideoParameters):
        command = command["video"]
    else:
        command = command["audio"]

    ffmpeg_command: List = [name]

    ffmpeg_command += command["start"]

    if name == "ffprobe":
        ffmpeg_command += [
            "-v",
            "error",
            "-show_entries",
            "stream=width,height,codec_type,codec_name",
            "-of",
            "json",
        ]

    if before_commands:
        ffmpeg_command += before_commands

    if headers is not None:
        for i in headers:
            ffmpeg_command.append("-headers")
            ffmpeg_command.append(f'"{i}: {headers[i]}"')

    ffmpeg_command += [
        "-y",
        "-nostdin",
        "-i",
        f'"{path}"' if name == "ffmpeg" else path,
    ]
    ffmpeg_command += command["mid"]

    if name == "ffmpeg":
        ffmpeg_command += _build_ffmpeg_options(stream_parameters)

    ffmpeg_command += command["end"]
    if name == "ffmpeg":
        ffmpeg_command.append("pipe:1")

    return ffmpeg_command


def _get_stream_params(command: str):
    arg_names = ["base", "audio", "video"]
    command_args: Dict = {arg: [] for arg in arg_names}
    current_arg = arg_names[0]

    for part in shlex.split(command):
        arg_name = part[2:]
        if arg_name in arg_names:
            current_arg = arg_name
        else:
            command_args[current_arg].append(part)
    command_args = {
        command: _extract_stream_params(command_args[command]) for command in command_args
    }

    for arg in arg_names[1:]:
        for x in command_args[arg_names[0]]:
            command_args[arg][x] += command_args[arg_names[0]][x]

    del command_args[arg_names[0]]

    return command_args


def _extract_stream_params(command: List[str]):
    arg_names = ["start", "mid", "end"]
    command_args: Dict = {arg: [] for arg in arg_names}
    current_arg = arg_names[0]

    for part in command:
        arg_name = part[3:]
        if arg_name in arg_names:
            current_arg = arg_name
        else:
            command_args[current_arg].append(part)

    return command_args


def _build_ffmpeg_options(
    stream_parameters: Union[AudioParameters, VideoParameters],
) -> List[str]:
    log_level = logging.getLogger().level
    ffmpeg_level = "info" if log_level == logging.DEBUG else "quiet"

    options = ["-v", ffmpeg_level, "-f"]

    if isinstance(stream_parameters, VideoParameters):
        options.extend(
            [
                "rawvideo",
                "-r",
                str(stream_parameters.frame_rate),
                "-pix_fmt",
                "yuv420p",
                "-vf",
                f"scale={stream_parameters.width}:{stream_parameters.height}",
            ]
        )
    else:
        options.extend(
            [
                "s16le",
                "-ac",
                str(stream_parameters.channels),
                "-ar",
                str(stream_parameters.bitrate),
            ]
        )

    return options



from deprecation import deprecated


@deprecated(
    deprecated_in="1.0.0.dev1",
    details="This enum is no longer supported.",
)
class StreamType:
    def __init__(self):
        self._stream_type = 0

    @property
    def live_stream(self):
        self._stream_type = 3
        return self

    @property
    def local_stream(self):
        self._stream_type = 10
        return self

    """
    *** Pulse Stream ***
    Send bytes like a pulsation, and this reduce the slice,
    because the slice is too heavy

    Support: LiveStream, LocalStream
    """

    @property
    def pulse_stream(self):
        self._stream_type = 4
        return self

    @property
    def stream_mode(self):
        return self._stream_type



import asyncio
import re
import sys
from datetime import date

import ntgcalls
from aiohttp import ClientConnectionError, ClientResponse, ClientSession

from .__version__ import __version__
from .version_manager import VersionManager


class PyTgCallsSession:
    notice_displayed = False

    async def start(self):
        if not self.notice_displayed:
            PyTgCallsSession.notice_displayed = True
            year = date.today().year
            print(
                f"PyTgCalls v{__version__} powered by "
                f"NTgCalls v{ntgcalls.__version__}, Copyright (C) "
                f"2021-{year} Laky-64 <https://github.com/Laky-64>\n"
                "Licensed under the terms of the GNU Lesser "
                "General Public License v3 or later (LGPLv3+)\n",
            )
            try:
                remote_stable_ver = await self._remote_version("master")
                remote_dev_ver = await self._remote_version("dev")
                remote_test_ver = remote_stable_ver + ".99"
                if VersionManager.version_tuple(__version__) > VersionManager.version_tuple(
                    remote_test_ver
                ):
                    remote_ver = remote_readable_ver = remote_dev_ver
                    my_ver = __version__
                else:
                    remote_readable_ver = remote_stable_ver
                    remote_ver = remote_test_ver
                    my_ver = __version__ + ".99"
                if VersionManager.version_tuple(remote_ver) > VersionManager.version_tuple(my_ver):
                    text = (
                        f"Update Available!\n"
                        f"New PyTgCalls v{remote_readable_ver} "
                        f"is now available!\n"
                    )
                    if not sys.platform.startswith("win"):
                        print(f"\033[93m{text}\033[0m")
                    else:
                        print(text)
            except asyncio.exceptions.TimeoutError:
                pass
            except ClientConnectionError:
                pass

    @staticmethod
    async def _remote_version(branch: str):
        async def get_async(url) -> str:
            session = ClientSession()
            try:
                response: ClientResponse = await session.get(url, timeout=5)
                result_text = await response.text()
                response.close()
                return result_text
            finally:
                await session.close()

        result = re.findall(
            "__version__ = '(.*?)'",
            (
                await get_async(
                    f"https://raw.githubusercontent.com/"
                    f"pytgcalls/pytgcalls/{branch}"
                    f"/pytgcalls/__version__.py",
                )
            ),
        )
        return result[0] if len(result) > 0 else __version__



import asyncio
import json


class Youtube:
    @staticmethod
    async def cookies_loads(file_json: str):
        if file_json:
            try:
                with open(file_json, "r") as f:
                    cookies = json.load(f)
                return cookies
            except FileNotFoundError:
                return []
            except json.JSONDecodeError:
                return []
        return []

    @staticmethod
    async def download(file_json: str, url: str, name_txt="youtube_cookies.txt"):
        with open(name_txt, 'w') as f:
            f.write("# Netscape HTTP Cookie File\n")
            f.write("# This is a generated file! Do not edit.\n\n")
            cookies = await Youtube.cookies_loads(file_json)
            for cookie in cookies:
                f.write("\t".join([
                    cookie.get('domain', ''),
                    'TRUE' if cookie.get('hostOnly', False) else 'FALSE',
                    cookie.get('path', '/'),
                    'TRUE' if cookie.get('secure', False) else 'FALSE',
                    str(int(cookie.get('expirationDate', 0))),
                    cookie.get('name', ''),
                    cookie.get('value', ''),
                ]) + "\n")

        cmd = f"yt-dlp --cookies {name_txt} {url}"
        process = await asyncio.create_subprocess_shell(
            cmd, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE
        )
        stdout, stderr = await process.communicate()
        e = stderr.decode()
        if not e:
            e = "No errors"
        o = stdout.decode()
        if not o:
            o = "No output"

        OUTPUT = ""
        OUTPUT += f"<b>Output</b>: \n<code>{o}</code>\n"
        OUTPUT += f"<b>Errors</b>: \n<code>{e}</code>"
        return OUTPUT



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

import requests


class Monitors:
    def __init__(self, api_key: str = None):
        self.api_key = api_key

    def _make_request(self, method: str, url: str, params: dict = None, json_data: dict = None):
        headers = {"accept": "application/json", "api-key": self.api_key}
        try:
            response = requests.request(
                method, url, headers=headers, params=params, json=json_data
            )
            return response.json()
        except requests.RequestException:
            pass

    def add_checking_monitor(self, url: str = None, is_monitor: bool = False):
        if is_monitor:
            url = "https://randydev-ryuzaki-api.hf.space/ryuzaki/new-monitor"
            params = {"url": url}
            return self._make_request("POST", url, params=params)
        else:
            raise ValueError("Error: is_monitor must be True")

    def get_monitors(self, is_show: bool = False):
        if is_show:
            url = "https://randydev-ryuzaki-api.hf.space/ryuzaki/get-monitors"
            return self._make_request("POST", url)
        else:
            raise ValueError("Error: is_show must be True")



import string
from math import ceil, log
from os import urandom


class Session:
    @staticmethod
    def generate_session_id(length: int = 21) -> str:
        alphabet = f"_-{string.digits}{string.ascii_letters}"
        alphabet_len = len(alphabet)

        mask = 1
        if alphabet_len > 1:
            mask = (2 << int(log(alphabet_len - 1) / log(2))) - 1
        step = int(ceil(1.6 * mask * length / alphabet_len))

        session_id = ""
        while True:
            random_bytes = bytearray(urandom(step))

            for i in range(step):
                random_byte = random_bytes[i] & mask
                if random_byte < alphabet_len:
                    if alphabet[random_byte]:
                        session_id += alphabet[random_byte]

                        if len(session_id) == length:
                            return session_id



from typing import Union

from ntgcalls import StreamStatus

from ...exceptions import GroupCallNotFound
from ...scaffold import Scaffold
from ...types.groups.group_call import GroupCall
from ...types.list import List


class CallHolder(Scaffold):
    PLAYING = 1
    PAUSED = 2
    IDLE = 3

    def __init__(self):
        super().__init__()
        self._conversions = {
            StreamStatus.Playing: self.PLAYING,
            StreamStatus.Paused: self.PAUSED,
            StreamStatus.Idling: self.IDLE,
        }

    @property
    def calls(self):
        calls_list: dict = self._binding.calls()
        return List([GroupCall(x, self._conversions[calls_list[x]]) for x in calls_list])

    @property
    def active_calls(self):
        calls_list: dict = self._binding.calls()
        return List(
            [
                GroupCall(x, self._conversions[calls_list[x]])
                for x in calls_list
                if calls_list[x] != StreamStatus.Idling
            ]
        )

    async def get_active_call(
        self,
        chat_id: Union[int, str],
    ):
        calls_list: dict = self._binding.calls()
        chat_id = await self._resolve_chat_id(chat_id)

        if chat_id in calls_list:
            if calls_list[chat_id] != StreamStatus.Idling:
                return GroupCall(
                    chat_id,
                    self._conversions[calls_list[chat_id]],
                )

        raise GroupCallNotFound(chat_id)

    async def get_call(
        self,
        chat_id: int,
    ):
        calls_list: dict = self._binding.calls()
        chat_id = await self._resolve_chat_id(chat_id)

        if chat_id in calls_list:
            return GroupCall(chat_id, self._conversions[calls_list[chat_id]])

        raise GroupCallNotFound(chat_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import base64
import json
import os
from base64 import b64decode
from base64 import b64decode as idk
from datetime import datetime as dt
from typing import Optional

import requests
from g4f.client import Client as Clients_g4f
from pyrogram import Client, filters
from pyrogram.types import Message

from RyuzakiLib.api.fullstack import FullStackDev
from RyuzakiLib.api.reqs import AsyicXSearcher
from RyuzakiLib.hackertools.blackbox import Blackbox

API_KEYS = "6398769dabd9fe0e49bedce0354b40a9b1a69d9594dc9d48c1d8a2a071c51e89"

owner_base = f"""
Your name is Randy Dev. A kind and friendly AI assistant that answers in
a short and concise answer. Give short step-by-step reasoning if required.

- Powered by @xtdevs on telegram
Today is {dt.now():%A %d %B %Y %H:%M}
"""

class RendyDevChat:
    @staticmethod
    async def chat_hacked(
        args: str = None,
        latest_model: str = "openai-latest",
        model_id: Optional[int] = None,
        user_id: Optional[int] = 0,
        mongo_url: Optional[str] = None,
        system_prompt: Optional[str] = owner_base,
        list_model_all: Optional[bool] = False
    ):
        if latest_model == "openai-v2":
            url = "https://randydev-ryuzaki-api.hf.space/ryuzaki/chatgpt-old"
            params = {"query": args}
            check_response = await AsyicXSearcher.search(
                url,
                post=True,
                re_json=True,
                json=params
            )
            return check_response["randydev"]["message"]
        elif latest_model == "blackbox":
            url = "https://randydev-ryuzaki-api.hf.space/ryuzaki/blackbox"
            params = {"query": args}
            check_response = await AsyicXSearcher.search(
                url,
                post=True,
                re_json=True,
                json=params
            )
            return check_response["randydev"]["message"]
        elif latest_model == "list-model":
            if list_model_all:
                text = """
                ```python
                openai-v2
                gemini-pro
                google-ai
                betagoogle-ai
                """
                return text
            else:
                return "you can check set list_model_all=True"
        elif latest_model == "gemini-pro":
            url = f"https://randydev-ryuzaki-api.hf.space/ryuzaki/gemini-ai-pro"
            payload = {
                "query": args,
                "mongo_url": mongo_url,
                "user_id": user_id,
                "is_multi_chat": True
            }
            headers = {"accept": "application/json", "api-key": API_KEYS}
            check_response = await AsyicXSearcher.search(
                url,
                post=True,
                re_json=True,
                headers=headers,
                json=payload
            )
            return check_response["randydev"]["message"]
        elif latest_model == "microsoft":
            url = "https://randydev-ryuzaki-api.hf.space/ryuzaki/faceai"
            payload = {
                "query": args,
                "clients_name": "microsoft/Phi-3-mini-4k-instruct",
            }
            headers = {"accept": "application/json"}
            check_response = await AsyicXSearcher.search(
                url,
                post=True,
                re_json=True,
                headers=headers,
                json=payload
            )
            return check_response["randydev"]["message"]
        elif latest_model == "gemma":
            url = "https://randydev-ryuzaki-api.hf.space/ryuzaki/faceai"
            payload = {
                "query": args,
                "clients_name": "google/gemma-1.1-7b-it",
            }
            headers = {"accept": "application/json"}
            check_response = await AsyicXSearcher.search(
                url,
                post=True,
                re_json=True,
                headers=headers,
                json=payload
            )
            return check_response["randydev"]["message"]
        elif latest_model == "mistralai":
            url = "https://randydev-ryuzaki-api.hf.space/ryuzaki/faceai"
            payload = {
                "query": args,
                "clients_name": "mistralai/Mixtral-8x7B-Instruct-v0.1",
            }
            headers = {"accept": "application/json"}
            check_response = await AsyicXSearcher.search(
                url,
                post=True,
                re_json=True,
                headers=headers,
                json=payload
            )
            return check_response["randydev"]["message"]
        elif latest_model == "faceh4":
            url = "https://randydev-ryuzaki-api.hf.space/ryuzaki/faceai"
            payload = {
                "query": args,
                "clients_name": "HuggingFaceH4/zephyr-7b-beta",
            }
            headers = {"accept": "application/json"}
            check_response = await AsyicXSearcher.search(
                url,
                post=True,
                re_json=True,
                headers=headers,
                json=payload
            )
            return check_response["randydev"]["message"]
        elif latest_model == "google-ai":
            url = f"https://randydev-ryuzaki-api.hf.space/ryuzaki/google-ai"
            headers = {"accept": "application/json", "api-key": API_KEYS}
            params = {"query": args}
            check_response = await AsyicXSearcher.search(
                url,
                post=True,
                re_json=True,
                headers=headers,
                json=params
            )
            return check_response["randydev"]["message"]
        elif latest_model == "betagoogle-ai":
            url = f"https://randydev-ryuzaki-api.hf.space/ryuzaki/v1beta2-google-ai"
            headers = {"accept": "application/json", "api-key": API_KEYS}
            params = {"query": args}
            check_response = await AsyicXSearcher.search(
                url,
                post=True,
                re_json=True,
                headers=headers,
                json=params
            )
            return check_response["randydev"]["message"]
        elif latest_model == "gpt-4-turbo":
            url = f"https://randydev-ryuzaki-api.hf.space/ryuzaki/chatgpt-custom"
            headers = {"accept": "application/json", "api-key": API_KEYS}
            params = {"query": args, "model": "gpt-4-turbo"}
            check_response = await AsyicXSearcher.search(
                url,
                post=True,
                re_json=True,
                headers=headers,
                json=params
            )
            return check_response["randydev"]["message"]
        elif latest_model == "gpt-4o":
            clients_x = Clients_g4f()
            response = clients_x.chat.completions.create(
                model="gpt-4o",
                messages=[
                    {"role": "system", "content": system_prompt},
                    {"role": "user", "content": args}
                ],
            )
            return response.choices[0].message.content

    @staticmethod
    def download_images(image_urls):
        downloaded_paths = []
        for i, url in enumerate(image_urls, start=1):
            filename = f"image_{i}.png"
            FullStackDev.fast(url, filename=filename)
        downloaded_paths.append(filename)
        return downloaded_paths

    @staticmethod
    async def image_generator(args):
        url = f"https://randydev-ryuzaki-api.hf.space/ryuzaki/dalle3xl"
        headers = {"accept": "application/json", "api-key": API_KEYS}
        payload = {"query": args}
        check_response = await AsyicXSearcher.search(
            url,
            post=True,
            re_json=True,
            headers=headers,
            json=payload
        )
        return check_response



import json
import re
from typing import Dict, List, Optional

import requests
from pymongo import MongoClient


class FarFalle:
    """A class to interact with the FarFalle chat service.

    Attributes:
        session: A requests.Session object for making HTTP requests.
        url: The URL endpoint for the FarFalle chat service.
    """

    def __init__(
        self,
        mongo_url: Optional[str] = None,
        user_id: Optional[int] = None
    ):
        self.mongo_url = mongo_url
        self.user_id = user_id
        self.client = MongoClient(self.mongo_url)
        self.db = self.client.tiktokbot
        self.collection = self.db.users
        """Initializes the FarFalle class with a session and the service URL."""
        self.session: requests.Session = requests.Session()
        self.url: str = "https://farfalle.onrender.com/chat"

    def _close(self):
        self.client.close()

    def _get_falle_chat_from_db(self):
        user_data = self.collection.find_one({"user_id": self.user_id})
        return user_data.get("falle_chat", []) if user_data else []

    def _update_falle_chat_in_db(self, falle_chat):
        self.collection.update_one(
            {"user_id": self.user_id},
            {"$set": {"falle_chat": falle_chat}},
            upsert=True
        )

    def _clear_history_in_db(self):
        unset_clear = {"falle_chat": None}
        return self.collection.update_one({"user_id": self.user_id}, {"$unset": unset_clear})

    def clear_database(self):
        """Clear the history for the current user."""
        result = self._clear_history_in_db()
        if result.modified_count > 0:
            return "Chat history cleared successfully."
        else:
            return "No chat history found to clear."

    def generate(self, query, model: Optional[str] = 'llama-3-70b', stream: Optional[bool] = False):
        """Generates a response from the FarFalle chat service.

        Args:
            conversation_history: A list of dictionaries containing the conversation history.
            model: The model name to use for generating the response. Defaults to 'llama-3-70b'.
            stream: If True, prints the content as it arrives. Defaults to False.

        Available models: 'llama-3-70b', 'gpt-3.5-turbo', 'gpt-4o'.

        Returns:
            A tuple containing the generated response as a string and the sources as a dictionary.
        """
        conversation_history = self._get_falle_chat_from_db()
        # Prepare the payload with the extracted query and the updated conversation_history
        payload = {
            "query": query,
            "history": conversation_history,
            "model": model
        }

        content = ""
        sources = {}
        # Make the POST request and stream the response
        response = self.session.post(self.url, json=payload, stream=True)
        for line in response.iter_lines(decode_unicode=True, chunk_size=1):
            if line:
                modified_line = re.sub("data:", "", line)
                try:
                    json_data = json.loads(modified_line)
                    if not sources:
                        sources = json_data
                    content += json_data['data']['text']
                    if stream:
                        print(json_data['data']['text'], end="", flush=True)
                except: continue
        conversation_history.append({"role": "assistant", "content": content})
        self._update_falle_chat_in_db(conversation_history)
        return content, sources

    def process_sources(self, sources: dict) -> tuple[List[Dict[str, str]], List[str]]:
        """Processes the sources dictionary to extract relevant information.

        Args:
            sources: A dictionary containing the sources data.

        Returns:
            A tuple containing a list of dictionaries with title, url, and content for each result,
            and a list of image URLs.
        """
        results = []
        images = []

        for result in sources['data']['results']:
            results.append({
                'title': result['title'],
                'url': result['url'],
                'content': result['content']
            })

        images = sources['data']['images']

        return results, images



import logging
from math import floor
from time import time
from typing import Any, List, Optional

from ..types import Cache
from ..types.groups import GroupCallParticipant
from ..types.participant_list import ParticipantList
from .bridged_client import BridgedClient

py_logger = logging.getLogger("pytgcalls")


class ClientCache:
    def __init__(
        self,
        cache_duration: int,
        app: BridgedClient,
    ):
        self._app: BridgedClient = app
        self._cache_duration = cache_duration
        self._full_chat_cache = Cache()
        self._call_participants_cache = Cache()

    async def get_full_chat(
        self,
        chat_id: int,
    ) -> Optional[Any]:
        full_chat = self._full_chat_cache.get(chat_id)
        if full_chat is not None:
            py_logger.debug("FullChat cache hit for %d", chat_id)
            return full_chat
        else:
            # noinspection PyBroadException
            try:
                py_logger.debug("FullChat cache miss for %d", chat_id)
                full_chat = await self._app.get_call(chat_id)
                self.set_cache(
                    chat_id,
                    full_chat,
                )
                return full_chat
            except Exception:
                pass
        return None

    def set_participants_cache(
        self,
        input_id: int,
        user_id: int,
        muted: Optional[bool],
        volume: Optional[int],
        can_self_unmute: Optional[bool],
        video: Optional[bool],
        screen_sharing: Optional[bool],
        video_camera: Optional[bool],
        raised_hand: Optional[int],
        left: Optional[int],
    ) -> Optional[GroupCallParticipant]:
        chat_id = self.get_chat_id(input_id)
        if chat_id is not None:
            participants: Optional[ParticipantList] = self._call_participants_cache.get(
                chat_id,
            )
            if participants is not None:
                if not left:
                    return participants.set_participant(
                        user_id,
                        muted if muted is not None else False,
                        muted != can_self_unmute,
                        video if video is not None else False,
                        screen_sharing if screen_sharing is not None else False,
                        video_camera if video_camera is not None else False,
                        raised_hand is not None,
                        floor(volume / 100) if volume is not None else 100,
                    )
                else:
                    return participants.remove_participant(
                        user_id,
                        muted if muted is not None else False,
                        muted != can_self_unmute,
                        video if video is not None else False,
                        screen_sharing if screen_sharing is not None else False,
                        video_camera if video_camera is not None else False,
                        raised_hand is not None,
                        floor(volume / 100) if volume is not None else 100,
                    )
        return None

    async def get_participant_list(
        self,
        chat_id: int,
    ) -> Optional[List[GroupCallParticipant]]:
        input_call = await self.get_full_chat(
            chat_id,
        )
        if input_call is not None:
            participants: Optional[ParticipantList] = self._call_participants_cache.get(
                chat_id,
            )
            if participants is not None:
                last_update = participants.last_mtproto_update
                curr_time = int(time())
                if not (last_update - curr_time > 0):
                    py_logger.debug(
                        "GetParticipant cache miss for %d",
                        chat_id,
                    )
                    try:
                        list_participants = await self._app.get_participants(
                            input_call,
                        )
                        for participant in list_participants:
                            self.set_participants_cache(
                                input_call.id,
                                participant["user_id"],
                                participant["muted"],
                                participant["volume"],
                                participant["can_self_unmute"],
                                participant["video"] is not None
                                or participant["presentation"] is not None,
                                participant["presentation"] is not None,
                                participant["video"] is not None,
                                participant["raise_hand_rating"],
                                participant["left"],
                            )
                        participants.last_mtproto_update = curr_time + self._cache_duration
                    except Exception as e:
                        py_logger.error("Error for %s in %d", e, chat_id)
                else:
                    py_logger.debug("GetParticipant cache hit for %d", chat_id)
                return participants.get_participants()
        return []

    def get_chat_id(
        self,
        input_group_call_id: int,
    ) -> Optional[int]:
        for key in self._call_participants_cache.keys():
            participants = self._call_participants_cache.get(key)
            if participants is not None:
                if participants.input_id == input_group_call_id:
                    return key
        return None

    def set_cache(
        self,
        chat_id: int,
        input_call: Any,
    ) -> None:
        self._full_chat_cache.put(
            chat_id,
            input_call,
            self._cache_duration,
        )
        self._call_participants_cache.put(
            chat_id,
            ParticipantList(
                input_call.id,
            ),
        )

    def drop_cache(
        self,
        chat_id,
    ) -> None:
        self._full_chat_cache.pop(chat_id)
        self._call_participants_cache.pop(chat_id)



from ..py_object import PyObject
from .audio_quality import AudioQuality


class AudioParameters(PyObject):
    def __init__(
        self,
        bitrate: int = 48000,
        channels: int = 1,
    ):
        max_bit, max_chan = max(AudioQuality, key=lambda x: x.value[0]).value
        self.bitrate: int = min(bitrate, max_bit)
        self.channels: int = min(channels, max_chan)

    @staticmethod
    def from_quality(quality: AudioQuality):
        return AudioParameters(*quality.value)



from typing import Union

from ...mtproto import BridgedClient
from ...scaffold import Scaffold


class ResolveChatID(Scaffold):
    async def _resolve_chat_id(self, chat_id: Union[int, str]) -> int:
        try:
            chat_id = int(chat_id)
        except ValueError:
            chat_id = BridgedClient.chat_id(
                await self._app.resolve_peer(chat_id),
            )

        return chat_id



from typing import Union

from RyuzakiLib.types.py_object import PyObject


class Update(PyObject):
    def __init__(
        self,
        chat_id: Union[int, str],
    ):
        self.chat_id = chat_id



import asyncio
import importlib

from pyrogram import Client, idle

from buildbot import client


async def start_bot():
    try:
        await client.start()
        ur = await client.get_me()
        print(f"Started {ur.first_name}")
    except Exception as e:
        print(f"{e}")
    await idle()


loop = asyncio.get_event_loop()
loop.run_until_complete(start_bot())



from dataclasses import dataclass
from time import time
from typing import Any, Dict, Optional


@dataclass
class CacheEntry:
    time: int
    expiry_time: int
    data: Any


class Cache:
    def __init__(self):
        self._store: Dict[int, CacheEntry] = {}

    def get(self, chat_id: int) -> Optional[Any]:
        if chat_id in self._store:
            if self._store[chat_id].time == 0 or self._store[chat_id].time - int(time()) > 0:
                return self._store[chat_id].data
            else:
                self._store.pop(chat_id, None)
        return None

    def put(self, chat_id: int, data: Any, expiry_time: int = 0) -> None:
        self._store[chat_id] = CacheEntry(
            time=0 if expiry_time == 0 else (int(time()) + expiry_time),
            expiry_time=expiry_time,
            data=data,
        )

    def keys(self):
        return [key for key in self._store]

    def pop(self, chat_id: int) -> Optional[Any]:
        return self._store.pop(chat_id, None)



import logging
from typing import Optional, Union

from ntgcalls import FileError

from ...exceptions import ClientNotStarted, NoMTProtoClientSet, NotInGroupCallError
from ...scaffold import Scaffold
from ...to_async import ToAsync
from ...types import ChangedStream
from ...types.input_stream.stream import Stream
from ..utilities.stream_params import StreamParams

py_logger = logging.getLogger("ryuzakilib")


class ChangeStream(Scaffold):
    async def change_stream(
        self,
        chat_id: Union[int, str],
        stream: Optional[Stream] = None,
    ):
        if self._app is None:
            raise NoMTProtoClientSet()

        if not self._is_running:
            raise ClientNotStarted()

        chat_id = await self._resolve_chat_id(chat_id)
        try:
            await ToAsync(
                self._binding.change_stream,
                chat_id,
                await StreamParams.get_stream_params(stream),
            )
        except FileError:
            raise FileNotFoundError()
        except Exception:
            raise NotInGroupCallError()

        await self._on_event_update.propagate(
            "RAW_UPDATE_HANDLER",
            self,
            ChangedStream(chat_id),
        )



from ...types.groups.group_call_participant import GroupCallParticipant
from ...types.update import Update


class UpdatedGroupCallParticipant(Update):
    def __init__(
        self,
        chat_id: int,
        participant: GroupCallParticipant,
    ):
        super().__init__(chat_id)
        self.participant = participant



from sys import platform


class DeviceInfo:
    def __init__(
        self,
        identifier: str,
        title: str,
    ):
        self.identifier = identifier
        self.title = title
        self.ffmpeg_parameters = ["-f"]

    def build_ffmpeg_command(self):
        if platform == "win32":
            self.ffmpeg_parameters += ["dshow"]
            return f"audio={self.identifier}"
        else:
            self.ffmpeg_parameters += ["pulse"]
        return self.identifier



from typing import Dict, List, Optional, Tuple

from .audio_parameters import AudioParameters
from .stream import Stream
from .video_parameters import VideoParameters


class SmartStream(Stream):
    def __init(self):
        self._audio_data: Tuple[
            str,
            str,
            AudioParameters,
            List[str],
            Optional[Dict[str, str]],
        ]
        self._video_data: Tuple[
            str,
            str,
            VideoParameters,
            List[str],
            Optional[Dict[str, str]],
        ]
        pass

    async def check_stream(self):
        pass



import os
import re

import setuptools


def read(fname, version=False):
    text = open(os.path.join(os.path.dirname(__file__), fname), encoding="utf8").read()
    if version:
        return re.search(r'__version__ = "(.*?)"', text).group(1)
    return text


setuptools.setup(
    name="RyuzakiLib",
    packages=setuptools.find_packages(),
    version=read("RyuzakiLib/__version__.py", version=True),
    license="MIT",
    description="RyuzakiLib Python Wrapper For API etc",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="TeamKillerX",
    author_email="killerx@randydev.my.id",
    url="https://github.com/TeamKillerX/RyuzakiLib/",
    keywords=["API", "RyuzakiLib", "Ryuzaki_Library", "Ryuzaki-API"],
    install_requires=[
        "requests",
        "aiohttp",
        "aiofiles",
        "pyrogram",
        "bs4",
        "google-search-results",
        "httpx[http2]",
        "Python-IO",
        "Pillow",
        "openai",
        "tqdm",
        "pre-commit",
        "gpytranslate",
        "Flask",
        "pymongo",
        "pytz",
        "urllib3",
        "pydantic",
        "typing-extensions",
        "typing",
        "deprecation",
        "ntgcalls>=1.0.2",
        "psutil",
        "screeninfo",
        "g4f",
        "curl_cffi",
        "google-generativeai",
        "gradio-client",
        "gradio",
        "huggingface_hub==0.22.2",
        "motor",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Natural Language :: English",
    ],
    python_requires="~=3.7",
)



import json
from typing import Any, Dict, List, Literal, Optional, Union

import aiofiles
import aiohttp


class Jiosaavn:
    """
    A class to interact with the JioSaavn API for searching and downloading songs, albums, artists, and playlists.
    """

    BASE_URL = "https://www.jiosaavn.com"
    API_URL = f"{BASE_URL}/api.php"

    async def _request_data(
        self,
        url: str,
        params: Dict[str, Any]
    ) -> Union[Dict[str, Any], List[Any]]:
        """
        Makes an asynchronous GET request to the specified URL with the given parameters.

        Args:
            url (str): The URL to send the GET request to.
            params (Dict[str, Any]): The query parameters for the GET request.

        Returns:
            Union[Dict[str, Any], List[Any]]: The JSON response from the request.

        Raises:
            RuntimeError: If there is an error during the request or if the response cannot be decoded as JSON.
        """
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url=url, params=params) as response:
                    response.raise_for_status()  # Raise an exception for HTTP errors
                    response_text = await response.text()
                    return json.loads(response_text)
        except aiohttp.ClientError as e:
            raise RuntimeError(f"Request to {url} failed: {e}")
        except ValueError as e:
            raise RuntimeError(f"Failed to decode JSON response from {url}: {e}")

    async def search(
        self,
        query: str,
        search_type: Literal["songs", "albums", "artists", "playlists"],
        page_no: Optional[int] = 1,
        page_size: Optional[int] = 10
    ) -> Dict[str, Any]:
        """
        Search for songs, albums, artists, or playlists.

        Args:
            query (str): The search query string.
            search_type (Literal["songs", "albums", "artists", "playlists"]): The type of search to perform.
            page_no (Optional[int]): The page number for paginated results. Defaults to 1.
            page_size (Optional[int]): The number of results per page. Defaults to 10.

        Returns:
            Dict[str, Any]: The search results from the API.

        Raises:
            ValueError: If `page_no` or `page_size` are not positive integers, or if `search_type` is invalid.
        """
        if page_no is not None and page_no < 1:
            raise ValueError("`page_no` must be a positive integer.")
        if page_size is not None and page_size < 1:
            raise ValueError("`page_size` must be a positive integer.")

        search_type_call_map = {
            "songs": "search.getResults",
            "albums": "search.getAlbumResults",
            "artists": "search.getArtistResults",
            "playlists": "search.getPlaylistResults",
        }

        call = search_type_call_map.get(search_type)
        if not call:
            raise ValueError(f"Invalid search_type: {search_type}")

        params = {
            'p': page_no,
            'q': query,
            '__call': call,
            'api_version': 4,
            'n': page_size,
            '_format': 'json',
            '_marker': 0,
            'ctx': 'web6dot0'
        }

        return await self._request_data(self.API_URL, params=params)

    async def search_all_types(
        self,
        query: str
    ) -> Dict[str, Any]:
        """
        Retrieve search results across all types, returning a limited subset due to API constraints.

        Args:
            query (str): The search query string.

        Returns:
            Dict[str, Any]: The search results from the API.
        """
        params = {
            '_format': 'json',
            '_marker': 0,
            'query': query,
            '__call': 'autocomplete.get',
            'ctx': 'web6dot0'
        }
        return await self._request_data(self.API_URL, params=params)

    async def get_artist(
        self,
        artist_id: Optional[str] = None,
        page_no: Optional[int] = 1,
        page_size: Optional[int] = 10
    ) -> Optional[Dict[str, Any]]:
        """
        Retrieves the details of an artist based on the provided ID.

        Args:
            artist_id (Optional[str]): The unique identifier for the artist. Defaults to None.
            page_no (Optional[int]): The page number for paginated results. Defaults to 1.
            page_size (Optional[int]): The number of results per page. Defaults to 10.

        Returns:
            Optional[Dict[str, Any]]: The details from the API or None if no response.

        Raises:
            ValueError: If `page_no` or `page_size` are not positive integers.
        """
        if page_no < 1:
            raise ValueError("`page_no` must be a positive integer.")
        if page_size < 1:
            raise ValueError("`page_size` must be a positive integer.")

        params = {
            '__call': 'webapi.get',
            'token': artist_id,
            'type': "artist",
            'p': page_no,
            'n_song': page_size,
            'n_album': page_size,
            'includeMetaTags': 0,
            'ctx': 'web6dot0',
            'api_version': 4,
            '_format': 'json',
            '_marker': 0
        }

        response = await self._request_data(self.API_URL, params=params)
        if not response:
            return None

        start_index = (page_no - 1) * page_size
        end_index = page_no * page_size
        response["topSongs"] += response["topAlbums"]
        response['count'] = len(response["topSongs"])
        response['topSongs'] = response["topSongs"][start_index:end_index]
        return response

    async def get_playlist_or_album(
        self,
        album_id: Optional[str] = None,
        playlist_id: Optional[str] = None,
        page_no: Optional[int] = 1,
        page_size: Optional[int] = 10
    ) -> Optional[Dict[str, Any]]:
        """
        Retrieves the details of a playlist or album based on the provided ID.

        Args:
            album_id (Optional[str]): The unique identifier for the album. Defaults to None.
            playlist_id (Optional[str]): The unique identifier for the playlist. Defaults to None.
            page_no (Optional[int]): The page number for paginated results. Defaults to 1.
            page_size (Optional[int]): The number of results per page. Defaults to 10.

        Returns:
            Optional[Dict[str, Any]]: The details from the API or None if no response or empty list for albums.

        Raises:
            ValueError: If both `album_id` and `playlist_id` are None or if `page_no` or `page_size` are not positive integers.
        """
        if page_no < 1:
            raise ValueError("`page_no` must be a positive integer.")
        if page_size < 1:
            raise ValueError("`page_size` must be a positive integer.")
        if not album_id and not playlist_id:
            raise ValueError("Either `album_id` or `playlist_id` must be provided.")

        search_type = "album" if album_id else "playlist"
        token = album_id or playlist_id

        params = {
            '__call': 'webapi.get',
            'token': token,
            'type': search_type,
            'p': page_no,
            'n': page_size,
            'includeMetaTags': 0,
            'ctx': 'web6dot0',
            'api_version': 4,
            '_format': 'json',
            '_marker': 0
        }

        response = await self._request_data(self.API_URL, params=params)
        if not response:
            return None

        if search_type == "playlist":
            return response

        if not response.get("list"):
            return None

        start_index = (page_no - 1) * page_size
        end_index = page_no * page_size
        response['list'] = response["list"][start_index:end_index]
        more_info = response.get("more_info", {})
        album_url = more_info.get("album_url")

        if not album_url:
            return response

        response["perma_url"] = album_url
        return response

    async def get_song(
        self,
        song_id: str
    ) -> Dict[str, Any]:
        """
        Retrieves the details of a song based on the song ID.

        Args:
            song_id (str): The unique identifier for the song.

        Returns:
            Dict[str, Any]: The song details from the API.
        """
        params = {
            '__call': 'webapi.get',
            'token': song_id,
            'type': 'song',
            'includeMetaTags': 0,
            'ctx': 'web6dot0',
            'api_version': 4,
            '_format': 'json',
            '_marker': 0
        }
        return await self._request_data(self.API_URL, params=params)

    async def get_song_lyrics(
        self,
        lyrics_id: str
    ) -> Dict[str, Any]:
        """
        Retrieves the lyrics of a song based on the lyrics ID.

        Args:
            lyrics_id (str): The ID of the lyrics to retrieve.

        Returns:
            Dict[str, Any]: The lyrics details from the API.

        Raises:
            ValueError: If the lyrics ID is invalid or the lyrics could not be retrieved.
        """
        params = {
            '__call': 'lyrics.getLyrics',
            'lyrics_id': lyrics_id,
            'type': 'song',
            'includeMetaTags': 0,
            'ctx': 'web6dot0',
            'api_version': 4,
            '_format': 'json',
            '_marker': 0
        }
        try:
            response = await self._request_data(self.API_URL, params=params)
            if not response:
                raise ValueError("No response received from the API.")
            return response
        except Exception as e:
            raise ValueError(f"Failed to retrieve lyrics for ID {lyrics_id}: {e}")

    async def get_download_url(
        self,
        song_id: str,
        bitrate: Literal[160, 320]
    ) -> Optional[Dict[str, Any]]:
        """
        Retrieves the download URL for a song based on the song ID and bitrate.

        Args:
            song_id (str): The unique identifier for the song.
            bitrate (Literal[160, 320]): The desired bitrate for the download.

        Returns:
            Optional[Dict[str, Any]]: The download data containing the URL or None if not found.
        """
        song_response = await self.get_song(song_id=song_id)
        if song_response:
            encrypted_media_url = song_response.get("songs", [])[0].get("more_info", {}).get("encrypted_media_url")
            params = {
                "__call": 'song.generateAuthToken',
                "url": encrypted_media_url,
                "bitrate": bitrate,
                "api_version": 4,
                "_format": "json",
                "ctx": "wap6dot0",
                "_marker": 0
            }
            return await self._request_data(url=self.API_URL, params=params)
        return None

    async def download_song(
        self,
        song_id: str,
        bitrate: Literal[160, 320],
        download_location: str
    ) -> None:
        """
        Downloads a song based on the song ID and bitrate.

        Args:
            song_id (str): The unique identifier for the song.
            bitrate (Literal[160, 320]): The desired bitrate for the download.
            download_location (str): The file path where the song will be saved.

        Raises:
            ValueError: If the song download URL cannot be retrieved.
        """
        download_data = await self.get_download_url(song_id=song_id, bitrate=bitrate)
        if not download_data or not download_data.get("auth_url"):
            raise ValueError("Unable to retrieve the download URL for the song.")

        url = download_data["auth_url"]

        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                response.raise_for_status()
                async with aiofiles.open(download_location, "wb") as file:
                    while True:
                        chunk = await response.content.read(4 * 1024 * 1024)  # 4 MB chunk size
                        if not chunk:
                            break
                        await file.write(chunk)



from typing import Union

from ntgcalls import ConnectionError

from ...exceptions import (
    ClientNotStarted,
    NoActiveGroupCall,
    NoMTProtoClientSet,
    NotInGroupCallError,
)
from ...scaffold import Scaffold
from ...to_async import ToAsync
from ...types import LeftVoiceChat


class LeaveGroupCall(Scaffold):
    async def leave_group_call(
        self,
        chat_id: Union[int, str],
    ):
        if self._app is None:
            raise NoMTProtoClientSet()

        if not self._is_running:
            raise ClientNotStarted()

        chat_id = await self._resolve_chat_id(chat_id)
        chat_call = await self._app.get_full_chat(
            chat_id,
        )

        if chat_call is None:
            raise NoActiveGroupCall()

        await self._app.leave_group_call(
            chat_id,
        )

        try:
            await ToAsync(
                self._binding.stop,
                chat_id,
            )
        except ConnectionError:
            raise NotInGroupCallError()

        if chat_id in self._need_unmute:
            del self._need_unmute[chat_id]

        await self._on_event_update.propagate(
            "RAW_UPDATE_HANDLER",
            self,
            LeftVoiceChat(chat_id),
        )



from .already_joined import AlreadyJoined
from .error_during_join import ErrorDuringJoin
from .group_call import GroupCall
from .group_call_participant import GroupCallParticipant
from .joined_group_call_participant import JoinedGroupCallParticipant
from .joined_voice_chat import JoinedVoiceChat
from .left_group_call_participant import LeftGroupCallParticipant
from .left_voice_chat import LeftVoiceChat
from .muted_call import MutedCall
from .not_in_group_call import NotInGroupCall
from .updated_group_call_participant import UpdatedGroupCallParticipant
from .upgrade_needed import UpgradeNeeded

__all__ = (
    "AlreadyJoined",
    "ErrorDuringJoin",
    "GroupCall",
    "GroupCallParticipant",
    "JoinedGroupCallParticipant",
    "JoinedVoiceChat",
    "LeftGroupCallParticipant",
    "LeftVoiceChat",
    "NotInGroupCall",
    "UpdatedGroupCallParticipant",
    "UpgradeNeeded",
    "MutedCall",
)



from ntgcalls import InputMode

from .video_parameters import VideoParameters


class VideoStream:
    def __init__(
        self,
        input_mode: InputMode,
        path: str,
        parameters: VideoParameters = VideoParameters(),
    ):
        self.input_mode: InputMode = input_mode
        self.path: str = path
        self.parameters: VideoParameters = parameters
